package com.bocloud.fms.booter.config;

import com.bocloud.cmp.boot.interceptor.AccessInterceptor;
import com.bocloud.cmp.boot.interceptor.LogInterceptor;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * 拦截器配置类
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年3月30日
 */
@Component
public class InterceptorConfiguration implements WebMvcConfigurer {

    private final AccessInterceptor accessInterceptor;
    private final LogInterceptor logInterceptor;

    public InterceptorConfiguration(AccessInterceptor accessInterceptor, LogInterceptor logInterceptor) {
        this.accessInterceptor = accessInterceptor;
        this.logInterceptor = logInterceptor;
    }

    /**
     * 添加拦截器
     *
     * @param registry
     */
    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(accessInterceptor).addPathPatterns("/**").order(0).excludePathPatterns("/status",
                "/report/files/**", "/status/**", "/v1/reports/{id}", "/log", "/v1/icons", "/v1/logs", "/favicon.ico",
                "/doc.html", "/swagger-resources/**", "/swagger-ui.html/**", "/webjars/**", "/v2/**", "/error",
                "/v1/logview", "/v1/logzip");
        registry.addInterceptor(logInterceptor).addPathPatterns("/**").excludePathPatterns("/favicon.ico", "/doc.html",
                "/swagger-resources/**", "/swagger-ui.html/**", "/webjars/**", "/v2/**", "/error").order(1);
    }

}
