package com.bocloud.fms.entity;

import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Date;

/**
 * @author hufy 流程定义实体类
 * @since 1.0
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Table("wf_definition")
public class FlowDefinition extends Generic implements Serializable, Cloneable {

    /**
     *
     */
    @IgnoreAll
    private static final long serialVersionUID = 6541688543201014542L;
    /**
     * 主键ID
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;

    /**
     * 流程定义名称
     */
    @Column("name")
    private String name;

    /**
     * 流程定义类型
     */
    @Column("type")
    private String type;

    /**
     * 当前流程的实例url（一般为流程第一步的url） 该字段可以直接打开流程申请的表单
     */
    @Column("instance_url")
    private String instanceUrl;

    /**
     * 是否可用的开关 0 非活动 1 活动
     */
    @Column("state")
    private String state;

    /**
     * 流程定义json
     */
    @Column("graph")
    private String graph;
    /**
     * 版本
     */
    @Column("version")
    private Integer version;

    /**
     * 创建时间
     */
    @Column("gmt_create")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;

    /**
     * 修改时间
     */
    @Column("gmt_modify")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;
    /**
     * 创建人
     */
    @Column("creator_id")
    private Long creatorId;
    /**
     * 创建人类型
     */
    @Column("creator_catalog")
    private RequestContext.Catalog creatorCatalog;
    /**
     * 修改人
     */
    @Column("mender_id")
    private Long menderId;
    /**
     * 修改人类型
     */
    @Column("mender_catalog")
    private RequestContext.Catalog menderCatalog;

    /**
     * 是否已删除 0 为删除 1 已删除
     */
    @Column("is_deleted")
    private Boolean deleted;

    /**
     * 最早的流程定义id ,如果是第一代，为空
     */
    @Column("baseline_id")
    private Long baseLineId;

    @Column("remark")
    private String remark;

    @Column("tenant_id")
    private long tenantId;

    @Column("operation_remark")
    private String operationRemark;


    /**
     * 创建者名称
     */
    @IgnoreAll
    private String creatorName;
    /**
     * 修改人名称
     */
    @IgnoreAll
    private String menderName;

    @Override
    public FlowDefinition clone() {
        try {
            return (FlowDefinition) super.clone();
        } catch (CloneNotSupportedException e) {
            return new FlowDefinition();
        }
    }

}
