package com.bocloud.fms.entity;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.apache.commons.lang3.StringUtils;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 流程工作单实体类（一般称为流程实例）
 *
 * @author hufy
 * @since 1.0
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Table("wf_instance")
public class FlowInstance extends Generic implements Serializable {
    @IgnoreAll
    private static final long serialVersionUID = -8335779448165343933L;
    /**
     * 主键ID
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;

    @Column("name")
    private String name;

    /**
     * 流程实例为子流程时，该字段标识父流程实例ID
     */
    @Column("parent_id")
    private Long parentId;

    /**
     * 流程定义ID
     */
    @Column("def_id")
    private Long defId;

    /**
     * 流程实例创建者ID
     */
    @Column("creator_id")
    private Long creatorId;

    /**
     * 流程实例创建者用户类型
     */
    @Column("creator_catalog")
    private RequestContext.Catalog creatorCatalog;

    /**
     * 流程实例创建时间
     */
    @Column("gmt_create")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;

    /**
     * 流程实例失效时间
     */
    @Column("gmt_expire")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtExpire;

    /**
     * 流程实例更新时间
     */
    @Column("gmt_update")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtUpdate;

    /**
     * 修改人id
     */
    @Column("mender_id")
    private Long menderId;

    /**
     * 流程实例修改者用户类型
     */
    @Column("mender_catalog")
    private RequestContext.Catalog menderCatalog;

    /**
     * 流程实例为子流程时，该字段标识父流程哪个节点模型启动的子流程
     */
    @Column("parent_node_name")
    private String parentNodeName;

    @Column("instance_no")
    private String instanceNo;
    /**
     * 流程实例附属变量
     */
    @Column("variable")
    private String variable;

    /**
     * verion
     */
    @Column("version")
    private Integer version;

    @Column("is_deleted")
    private Boolean deleted;

    @IgnoreAll
    private Map<String, Object> variableMap;

    @Column("instance_state")
    private String instanceState;

    @Column("remark")
    private String remark;

    @Column("gmt_finish")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtFinish;

    @IgnoreAll
    private String creatorName;
    @IgnoreAll
    private String menderName;

    @Column("tenant_id")
    private Long tenantId;

    /**
     * 项目（信息系统）id
     */
    @Column("project_id")
    private Long projectId;
    @IgnoreAll
    private String projectName;
    // 申请类型
    @Column("apply_category")
    private String applyCategory;
    // 执行结果（针对需要资源实施的审批）
    @Column("execute_result")
    private String executeResult;
    // 审核状态
    @IgnoreAll
    private String applyState;
    // 审批人
    @IgnoreAll
    private List<Object> actors;
    // 等待时长
    @IgnoreAll
    private String duration;
    // 当前审批人，如果是角色则显示角色名称
    @IgnoreAll
    private String currentAuditors;

    public Map<String, Object> getVariableMap() {
        if (variableMap == null || variableMap.size() == 0) {
            if (StringUtils.isNotBlank(variable)) {
                variableMap = JSONObject.parseObject(this.variable);
            }
        }
        return variableMap;
    }

}
