/* Copyright 2013-2015 www.snakerflow.com.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.bocloud.fms.entity;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author hufy 任务实体类
 * @since 1.0
 */
@Data
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@Table("wf_node")
public class FlowNode extends Generic implements Serializable, Cloneable {
    /**
     *
     */
    @IgnoreAll
    private static final long serialVersionUID = -189094546633914087L;
    @IgnoreAll
    public static final String KEY_ACTOR = "S-ACTOR";
    /**
     * 主键ID
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;

    /**
     * 编码
     */
    @Column("code")
    private String code;

    /**
     * 流程实例ID
     */
    @Column("instance_id")
    private Long instanceId;

    /**
     * 任务节点id
     */
    @Column("node_id")
    private String nodeId;

    /**
     * 任务名称
     */
    @Column("name")
    private String name;

    /**
     * 任务处理者ID
     */
    @Column("operator_id")
    private Long operatorId;

    /**
     * 任务处理者类型
     */
    @Column("operator_catalog")
    private RequestContext.Catalog operatorCatalog;

    /**
     * 流程实例创建时间
     */
    @Column("gmt_create")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;

    /**
     * 流程实例失效时间
     */
    @Column("gmt_expire")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtExpire;

    /**
     * 流程实例结束时间
     */
    @Column("gmt_finish")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtFinish;

    /**
     * 提醒时间date类型
     */
    @Column("gmt_remind")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtRemind;
    /**
     * 任务关联的表单url
     */
    @Column("action_url")
    private String actionUrl;

    /**
     * 父任务Id
     */
    @Column("parent_node_id")
    private Long parentNodeId;
    /**
     * 任务附属变量
     */
    @Column("variable")
    private String variable;

    /**
     * 版本
     */
    @Column("version")
    private Integer version = 0;

    @Column("is_deleted")
    private Boolean deleted;

    @Column("node_state")
    private String nodeState;

    /**
     * 附件組id
     */
    @Column("group_id")
    private String groupId;

    @IgnoreAll
    private String operatorName;

    @IgnoreAll
    private List<RequestContext> actors;
    @IgnoreAll
    private List<Attachment> attachments;

    /**
     * 下一参与者
     */
    @IgnoreAll
    private String nextActorNames;

    public List<RequestContext> getActors() {
        if (actors == null) {
            String actorStr = (String) getVariableMap().get(KEY_ACTOR);
            if (actorStr != null && actorStr.length() > 0) {
                actors = JSONArray.parseArray(actorStr, RequestContext.class);
            }
        }
        return actors;
    }

    public Map<String, Object> getVariableMap() {
        Map<String, Object> map = JSONObject.parseObject(this.variable);
        if (map == null) {
            return Collections.emptyMap();
        }
        return map;
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        return super.clone();
    }

}
