package com.bocloud.fms.entity;

import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.annotations.IgnoreAll;
import com.bocloud.entity.annotations.PK;
import com.bocloud.entity.annotations.Table;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

/**
 * 资源配额类
 *
 * @author wangyu
 * @since 2017年7月4日
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Table("resource_quota")
@EqualsAndHashCode(callSuper = true)
public class ResourceQuota extends Generic {

    /**
     * id
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;
    /**
     * 租户id
     */
    @Column("target")
    private Long target;
    /**
     * 总配额
     */
    @Column("quota")
    private Long quota;
    /**
     * 已使用配额量
     */
    @Column("used")
    private Long used;
    /**
     * 配额单位
     */
    @Column("unit")
    private String unit;
    /**
     * 配额编码
     */
    @Column("code")
    private String code;
    /**
     * 配额类型
     */
    @Column("category")
    private Category category;
    /**
     * 剩余配额量
     */
    @IgnoreAll
    private Long freeQuota;
    /**
     * 旧配额量
     */
    @IgnoreAll
    private Long oldQuota;
    /**
     * 最小可配额值
     */
    @IgnoreAll
    private Long minQuota;
    /**
     * 最大可配额值
     */
    @IgnoreAll
    private Long maxQuota;
    /**
     * 租户名称
     */
    @IgnoreAll
    private String tenantName;
    /**
     * 配额是否可以释放
     */
    @IgnoreAll
    private Boolean releasable;
    /**
     * 配额名称
     */
    @IgnoreAll
    private String name;
    /**
     * 配额操作值
     */
    @IgnoreAll
    private Long quotaActionValue;
    /**
     * 排序序号
     */
    @IgnoreAll
    private Long sn;

    public ResourceQuota(Long target, Long quota, Long used, String code, Category category) {
        this.target = target;

        this.quota = quota;
        this.used = used;
        this.code = code;
        this.category = category;
    }

    public ResourceQuota(Long target, Long quota, Long used, String unit, String code,
                         Category category, Long minQuota, Long maxQuota, String name) {
        this.target = target;
        this.quota = quota;
        this.used = used;
        this.unit = unit;
        this.code = code;
        this.category = category;
        this.minQuota = minQuota;
        this.maxQuota = maxQuota;
        this.name = name;
    }

    public ResourceQuota(Long target, Long quota, Long used, String unit, String code,
                         Category category) {
        this.target = target;
        this.quota = quota;
        this.used = used;
        this.code = code;
        this.unit = unit;
        this.category = category;
    }

    public ResourceQuota(String code, Long quotaActionValue) {
        this.code = code;
        this.quotaActionValue = quotaActionValue;
    }

    /**
     * 配额类型
     */
    public enum Category {
        /**
         * 服务配额
         */
        Service,
        /**
         * 资源配额
         */
        Meta,
    }

    public enum Action {
        /**
         * 分出(分配量增加)
         */
        Distribute,
        /**
         * 分得(总量增加)
         */
        Assign,
        /**
         * 订购(使用增加)
         */
        Order,
        /**
         * 减配(总量减少)
         */
        Decrease,
        /**
         * 释放(分配减少)
         */
        Release,
        /**
         * 回滚
         */
        Callback,
    }
}
