package com.bocloud.fms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.fms.entity.FlowInstance;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 流程工作单实体类（一般称为流程实例）
 *
 * @author weiwei
 */
@Repository("flowInstanceRepository")
public class FlowInstanceRepository extends BasicGenericDao<FlowInstance, Long> {

    public FlowInstanceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public FlowInstance query(Long id) {
        String sql = "select a.* from wf_instance a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<FlowInstance> list = this.list(FlowInstance.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public Long count(Pager pager) {
        List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
        boolean ifQueryOperator = params.stream().anyMatch(param -> param.getParam().containsKey("operatorId"));
        boolean ifQueryActor = params.stream().anyMatch(param -> param.getParam().containsKey("actorId"));

        StringBuilder sql = new StringBuilder("select count(distinct o.id) from wf_instance o ");
        sql.append(" left join wf_definition p on p.id = o.def_id ");
        if (ifQueryOperator) {
            // 排除起点的数据，fix bug PBSM-6956 2019年9月10日 13:58:55
            sql.append(" left join wf_node wt on o.id = wt.instance_id and (wt.code != 'DRAFT' or wt.code is null) ");
        }
        if (ifQueryActor) {
            if (ifQueryOperator) {
                sql.append(" left join wf_node_actor wta on wta.node_id = wt.id ");
            } else {
                sql.append(
                        "  left join wf_node wt on o.id = wt.instance_id left join wf_node_actor wta on wta.node_id = wt.id ");
            }
        }
        sql.append(" where 1=1 ");
        String querySql = QueryBuilder.buildRaw(sql.toString(), pager.getParams(), null, null);
        Map<String, Object> paramMap = QueryBuilder.getParam(pager.getParams());
        return super.countQuery(querySql, paramMap);
    }

    public List<FlowInstance> getFlowInstances(Pager pager) {
        List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
        boolean ifQueryOperator = params.stream().anyMatch(param -> param.getParam().containsKey("operatorId"));
        boolean ifQueryActor = params.stream().anyMatch(param -> param.getParam().containsKey("actorId"));

        StringBuilder sql = new StringBuilder("select  o.* from wf_instance o ");
        sql.append(" left join wf_definition p on p.id = o.def_id ");
        if (ifQueryOperator) {
            // 排除起点的数据，fix bug PBSM-6956 2019年9月10日 13:58:55
            sql.append(" left join wf_node wt on o.id = wt.instance_id and (wt.code != 'DRAFT' or wt.code is null) ");
        }
        if (ifQueryActor) {
            if (ifQueryOperator) {
                sql.append(" left join wf_node_actor wta on wta.node_id = wt.id ");
            } else {
                sql.append(
                        "  left join wf_node wt on o.id = wt.instance_id left join wf_node_actor wta on wta.node_id = wt.id ");
            }
        }
        sql.append(" where 1=1 ");
        String querySql = QueryBuilder.buildRaw(sql.toString(), pager, "");
        Map<String, Object> paramMap = QueryBuilder.getParam(pager.getParams());
        return super.list(FlowInstance.class, querySql, paramMap);
    }

    public List<FlowInstance> list(int page, int rows, Map<String, String> sorter, List<Param> params, Long target,
                                   String category) {
        String sql = "select a.* from wf_instance a,flow_relation b where a.is_deleted = 0 and a.id = b.instance_id and b.target = :target and b.category = :category";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("target", target);
        paramMap.put("category", category);
        return this.list(FlowInstance.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, Long target, String category) {
        String sql = "select a.* from wf_instance a,flow_relation b where a.is_deleted = 0 and a.id = b.instance_id and b.target = :target and b.category = :category";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("target", target);
        paramMap.put("category", category);
        List<FlowInstance> list = this.list(FlowInstance.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (FlowInstance obj : list) {
            beans.add(new SimpleBean(obj.getId(), obj.getName()));
        }
        return beans;
    }

    public Integer count(List<Param> params, Long target, String category) {
        String sql = "select count(a.id) from wf_instance a,flow_relation b where a.is_deleted = 0 and a.id = b.instance_id and b.target = :target and b.category = :category";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("target", target);
        paramMap.put("category", category);
        return this.countQuery(sql, paramMap).intValue();
    }

}
