package com.bocloud.fms.engine;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.fms.engine.cfg.FlowConfiguration;
import com.bocloud.fms.entity.Attachment;
import com.bocloud.fms.intf.FlowDefinitionService;
import com.bocloud.fms.intf.FlowInstanceService;
import com.bocloud.fms.intf.FlowNodeService;
import com.bocloud.fms.intf.FlowQueryService;
import com.bocloud.fms.service.Execution;

import java.util.List;
import java.util.Map;

/**
 * 流程引擎接口
 *
 * @author hufy
 * @since 1.0
 */
public interface FlowEngine {

    /**
     * 根据Configuration对象配置实现类
     *
     * @param config 全局配置对象
     * @return FlowEngine 流程引擎
     */
    FlowEngine configure(FlowConfiguration config);

    /**
     * 获取definition服务
     *
     * @return IDefinitionService 流程定义服务
     */
    FlowDefinitionService definition();

    /**
     * 获取查询服务
     *
     * @return IQueryService 常用查询服务
     */
    FlowQueryService query();

    /**
     * 获取实例服务
     *
     * @return IQueryService 流程实例服务
     */
    FlowInstanceService flowInstance();

    /**
     * 获取任务服务
     *
     * @return ITaskService 任务服务
     */
    FlowNodeService node();

    /**
     * 分配参与者的处理接口
     *
     * @return
     */
    AssignmentHandler assignmentHandler();

    /**
     * 根据流程定义ID，操作人ID，参数列表启动流程实例
     *
     * @param id      流程定义ID
     * @param context 操作人ID
     * @param project projectID
     * @param args    参数列表
     * @return execution
     */
    Execution startInstanceById(Long id, RequestContext context, Long project, Map<String, Object> args);

    /**
     * 根据任务主键ID，操作人ID，参数列表执行任务
     *
     * @param taskId      任务主键ID
     * @param context     当前用户
     * @param args        参数列表
     * @param attachments 附件
     * @return
     */
    BocloudResult executeNode(Long taskId, RequestContext context, Map<String, Object> args, List<Attachment> attachments);

    /**
     * 批量审批（仅适用于当前用户当前审批节点个数为1的情况）
     *
     * @param instanceIds 流程实例ID集合
     * @param context     当前用户
     * @param args        参数列表
     * @param attachments 附件
     * @return
     */
    BocloudResult batchExecuteNode(List<Long> instanceIds, RequestContext context, Map<String, Object> args, List<Attachment> attachments);

}
