package com.bocloud.fms.engine;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.fms.engine.model.FlowNodeModel;
import com.bocloud.fms.service.internal.SmsProjectService;
import com.bocloud.fms.service.internal.SmsRoleService;
import com.bocloud.fms.service.internal.SmsTenantService;
import com.bocloud.fms.internel.entity.Tenant;
import com.bocloud.fms.internel.entity.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * 分配参与者的处理抽象类
 *
 * @author hufy
 */
@Component
public class GenericAssignment implements AssignmentHandler {

    @Autowired
    private SmsTenantService smsTenantService;
    @Autowired
    private SmsProjectService smsProjectService;
    @Autowired
    private SmsRoleService smsRoleService;

    @Override
    public Set<RequestContext> getAssignees(FlowNodeModel model, Long projectId, RequestContext context) {
        Set<RequestContext> assignees = new HashSet<>();
        // 特定人
        if (null != model.getAssignee()) {
            model.getAssignee().forEach(id -> assignees.add(new RequestContext(RequestContext.Catalog.User, id, context.getTenant(), null, null)));
        }
        // 角色
        if (null != model.getAssigneeRole() && model.getAssigneeRole().size() > 0) {
            model.getAssigneeRole().forEach(roleId -> {
                List<Param> params = new ArrayList<>();
                params.add(new Param(MapTools.simpleMap("status", "NORMAL"), Sign.EQ));
                BocloudResult result = smsRoleService.listUsers(roleId, new Pager(1, Integer.MAX_VALUE, params, null));
                List<User> users = ((JSONObject) result.getData()).getJSONArray(Common.ROWS).toJavaList(User.class);
                users = Optional.ofNullable(users).orElse(new ArrayList<>());
                users.forEach(u -> assignees.add(new RequestContext(RequestContext.Catalog.User, u.getId(), context.getTenant(), null, null)));
            });
        }
        // 租户管理员
        if (model.getAssigneeTenantAdmin() != null && model.getAssigneeTenantAdmin() && 0L != context.getTenant()) {
            Tenant tenant = smsTenantService.detail(context.getTenant());
            assignees.add(new RequestContext(RequestContext.Catalog.Tenant, tenant.getId(), tenant.getId(), null, null));
        }
        // 项目经理
        if (model.getAssigneeProjectManager() != null && model.getAssigneeProjectManager() && projectId != null && 0L != projectId) {
            BocloudResult result = smsProjectService.listManagers(projectId);
            List<User> users = ((JSONArray) result.getData()).toJavaList(User.class);
            users = Optional.ofNullable(users).orElse(new ArrayList<>());
            users.forEach(u -> assignees.add(new RequestContext(RequestContext.Catalog.User, u.getId(), context.getTenant(), null, null)));
        }
        return assignees;
    }

}
