package com.bocloud.fms.engine;

import com.bocloud.fms.entity.FlowSequence;
import com.bocloud.fms.repository.FlowSequenceRepository;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * 工单编号生成器
 *
 * @author zhangyf
 * @since 2019-02-13
 */
@Component
public class InstanceNoGenerator {

    private Logger logger = LoggerFactory.getLogger(InstanceNoGenerator.class);

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private FlowSequenceRepository flowSequenceRepository;

    /**
     * 生成编号，格式:"业务类型前缀标识+日期+四位编号"，如201810170001
     *
     * @return
     */
    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    public String generate() {
        String result = null;
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        String time = sdf.format(date);
        String path = FlowSequence.class.getSimpleName() + "_" + time;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            FlowSequence flowSequence = flowSequenceRepository.query(time);
            if (null == flowSequence) {
                FlowSequence sequence = new FlowSequence();
                sequence.setDate(time);
                sequence.setSn("1");
                result = time + "0001";
                flowSequenceRepository.save(sequence);
            } else {
                int num = Integer.parseInt(flowSequence.getSn());
                flowSequence.setSn(String.valueOf(++num));
                flowSequenceRepository.update(flowSequence);
                String str = leftPad(num, 4, '0');
                result = time + str;
            }
        } catch (Exception e) {
            logger.error("save flow serial number error:", e);
        }
        return result;
    }

    /**
     * 0000-9999编码生成器
     *
     * @param num
     * @param maxLen
     * @param filledChar
     * @return
     */
    private static String leftPad(int num, final int maxLen, char filledChar) {
        StringBuffer sb = new StringBuffer();
        String str = String.valueOf(num);
        for (int i = str.length(); i < maxLen; i++) {
            sb.append(filledChar);
        }
        return sb.append(str).toString();
    }
}