/* Copyright 2013-2015 www.snakerflow.com.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.bocloud.fms.engine.cache.memory;

import com.bocloud.fms.common.utils.StringUtil;
import com.bocloud.fms.engine.cache.Cache;
import com.bocloud.fms.engine.cache.CacheException;
import com.bocloud.fms.engine.cache.CacheManager;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * 基于云主机内存的cache管理器
 *
 * @author hufy
 */
public class MemoryCacheManager<K, V> implements CacheManager {
    private final ConcurrentMap<String, Cache<K, V>> caches;

    public MemoryCacheManager() {
        this.caches = new ConcurrentHashMap<>();
    }

    @Override
    @SuppressWarnings("unchecked")
    public Cache<K, V> getCache(String name) throws CacheException {
        if (StringUtil.isEmpty(name)) {
            throw new IllegalArgumentException("Cache名称不能为空.");
        }
        Cache<K, V> cache;

        cache = caches.get(name);
        if (cache == null) {
            cache = new MemoryCache<>(new ConcurrentHashMap<>(100));
            Cache<K, V> existing = caches.putIfAbsent(name, cache);
            if (existing != null) {
                cache = existing;
            }
        }
        return cache;
    }

    @Override
    public void destroy() throws CacheException {
        while (!caches.isEmpty()) {
            caches.clear();
        }
    }
}
