/* Copyright 2013-2015 www.snakerflow.com.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.bocloud.fms.engine.cfg;

import com.bocloud.fms.common.exception.FlowException;
import com.bocloud.fms.common.utils.ConfigUtil;
import com.bocloud.fms.engine.Context;
import com.bocloud.fms.engine.FlowEngine;
import com.bocloud.fms.service.AbstractServiceContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Properties;

/**
 * @author hufy 只允许应用程序存在一个Configuration实例 初始化服务上下文，查找流程引擎实现类并初始化依赖的服务
 */
public class FlowConfiguration {
    /**
     *
     */
    private static final Logger log = LoggerFactory.getLogger(FlowConfiguration.class);

    /**
     * 根据服务查找实现类构造配置对象
     *
     * @param context 上下文实现
     */
    public FlowConfiguration(Context context) {
        AbstractServiceContext.setContext(context);
    }

    /**
     * 构造FlowEngine对象，用于api集成 通过SpringHelper调用
     *
     * @return FlowEngine
     * @throws FlowException
     */
    public FlowEngine buildFlowEngine() throws FlowException {
        if (log.isInfoEnabled()) {
            log.info("FlowEngine start......");
        }
        /**
         * 由服务上下文返回流程引擎
         */
        FlowEngine configEngine = AbstractServiceContext.getEngine();
        if (configEngine == null) {
            throw new FlowException("配置无法发现FlowEngine的实现类");
        }
        if (log.isInfoEnabled()) {
            log.info("FlowEngine be found:" + configEngine.getClass());
        }
        return configEngine.configure(this);
    }

    /**
     * 可装载自定义的属性配置文件
     *
     * @param fileName 属性文件名称
     * @return FlowConfiguration
     */
    public FlowConfiguration initProperties(String fileName) {
        ConfigUtil.loadProperties(fileName);
        return this;
    }

    /**
     * 可装载已有的属性对象
     *
     * @param properties 属性对象
     * @return FlowConfiguration
     */
    public FlowConfiguration initProperties(Properties properties) {
        ConfigUtil.loadProperties(properties);
        return this;
    }

}
