package com.bocloud.fms.engine.handlers.impl;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.Sign;
import com.bocloud.common.utils.MapTools;
import com.bocloud.fms.common.enums.FlowConstants;
import com.bocloud.fms.common.model.BasicDomainModel;
import com.bocloud.fms.engine.handlers.IHandler;
import com.bocloud.fms.engine.model.DefinitionModel;
import com.bocloud.fms.engine.model.FlowNodeModel;
import com.bocloud.fms.entity.FlowInstance;
import com.bocloud.fms.entity.FlowNode;
import com.bocloud.fms.intf.FlowQueryService;
import com.bocloud.fms.service.Execution;

import java.util.ArrayList;
import java.util.List;

/**
 * 合并处理的抽象处理器 需要子类提供查询无法合并的task集合的参数map
 *
 * @author hufy
 * @since 1.0
 */
public abstract class AbstractMergeHandler implements IHandler {
    @Override
    public void handle(Execution execution) {
        /**
         * 查询当前流程实例的无法参与合并的node列表 若所有中间node都完成，则设置为已合并状态，告诉model可继续执行join的输出变迁
         */
        FlowQueryService queryService = execution.getEngine().query();
        FlowInstance flowInstance = execution.getFlowInstance();
        DefinitionModel model = execution.getModel();
        String[] activeNodes = findActiveNodes();
        boolean isNodeMerged = false;

        if (model.containsNodeNames(FlowNodeModel.class, activeNodes)) {
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("t.instanceId", flowInstance.getId()), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("nodeState", FlowConstants.STATE_ACTIVE), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("nodeId", JSONObject.toJSONString(activeNodes).substring(1, JSONObject.toJSONString(activeNodes).length() - 1)), Sign.IN));
            params.add(new Param(MapTools.simpleMap("id", execution.getTask().getEntity().getId()), Sign.UEQ));
            List<BasicDomainModel<FlowNode, FlowNodeModel>> tasks = queryService.getActiveNodes(params);
            if (tasks == null || tasks.isEmpty()) {
                // 如果所有task都已完成，则表示可合并
                isNodeMerged = true;
            }
        }
        execution.setMerged(isNodeMerged && isOkToMerge(execution));
    }

    /**
     * 子类需要提供如何查询未合并任务的参数map
     *
     * @return
     */
    protected abstract String[] findActiveNodes();

    /**
     * 是否可以合并，子类实现
     *
     * @param execution 执行上下文
     * @return 返回是否可以合并
     */
    protected abstract Boolean isOkToMerge(Execution execution);
}
