package com.bocloud.fms.engine.handlers.impl;

import com.bocloud.common.model.RequestContext;
import com.bocloud.fms.common.exception.FlowException;
import com.bocloud.fms.common.model.BasicDomainModel;
import com.bocloud.fms.engine.FlowInterceptor;
import com.bocloud.fms.engine.handlers.IHandler;
import com.bocloud.fms.engine.model.FlowNodeModel;
import com.bocloud.fms.entity.FlowNode;
import com.bocloud.fms.service.AbstractServiceContext;
import com.bocloud.fms.service.Execution;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * 任务创建操作的处理器
 *
 * @author hufy
 */
public class CreateTaskHandler implements IHandler {
    private static final Logger log = LoggerFactory.getLogger(CreateTaskHandler.class);
    /**
     * 任务模型
     */
    private FlowNodeModel model;

    /**
     * 调用者需要提供任务模型
     *
     * @param model 模型
     */
    public CreateTaskHandler(FlowNodeModel model) {
        this.model = model;
    }

    /**
     * 根据任务模型、执行对象，创建下一个任务，并添加到execution对象的tasks集合中
     */
    @Override
    public void handle(Execution execution) {
        //创建节点
        List<BasicDomainModel<FlowNode, FlowNodeModel>> tasks = execution.getEngine().node().createTask(model,
                execution);
        execution.addTasks(tasks);
        /**
         * 从服务上下文中查找任务拦截器列表，依次对task集合进行拦截处理
         */
        List<FlowInterceptor> interceptors = AbstractServiceContext.getContext().findList(FlowInterceptor.class);
        try {
            for (FlowInterceptor interceptor : interceptors) {
                interceptor.intercept(execution);
            }
        } catch (Exception e) {
            log.error("拦截器执行失败=" + e.getMessage());
            throw new FlowException(e);
        }

        Optional.ofNullable(execution.getTasks()).orElse(new ArrayList<>()).stream().forEach(task -> {
            Set<RequestContext> actors = execution.getEngine().assignmentHandler().getAssignees(task.getModel(), execution.getFlowInstance().getProjectId(), execution.getContext());
            if (CollectionUtils.isEmpty(actors)) {
                Map<String, Object> params = new HashMap<>();
                params.put("auditFlag", "pass");
                params.put("remark", "未找到当前环节审批人，系统自动审批通过");
                params.put("id", task.getEntity().getId());
                RequestContext context = new RequestContext(RequestContext.Catalog.User, 1L, execution.getContext().getTenant(), 0L, null);
                execution.getEngine().executeNode(task.getEntity().getId(), context, params, null);
            } else if (null != task.getModel().getAutoExecute() && task.getModel().getAutoExecute()) {
                Map<String, Object> params = new HashMap<>(16);
                params.put("auditFlag", "pass");
                params.put("remark", "同意");
                params.put("id", task.getEntity().getId());
                if (CollectionUtils.isEmpty(actors)) {
                    actors.add(new RequestContext(RequestContext.Catalog.User, 1L, execution.getContext().getTenant(), 0L, null));
                }
                execution.getEngine().executeNode(task.getEntity().getId(), actors.stream().findFirst().get(), params, null);
            }
        });
    }
}
