package com.bocloud.fms.engine.handlers.impl;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.Sign;
import com.bocloud.common.utils.MapTools;
import com.bocloud.fms.common.enums.FlowConstants;
import com.bocloud.fms.engine.model.*;
import com.bocloud.fms.entity.FlowNode;
import com.bocloud.fms.intf.FlowQueryService;
import com.bocloud.fms.service.Execution;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 合并分支操作的处理器
 *
 * @author hufy
 */
public class MergeBranchHandler extends AbstractMergeHandler {
    private JoinModel model;

    public MergeBranchHandler(JoinModel model) {
        this.model = model;
    }

    /**
     * 对join节点的所有输入变迁进行递归，查找join至fork节点的所有中间task元素
     *
     * @param node
     * @param buffer
     */
    private void findForkNodeNames(BaseNodeModel node, StringBuilder buffer) {
        if (node instanceof ForkModel) {
            return;
        }
        List<TransitionModel> inputs = node.getInputs();
        for (TransitionModel tm : inputs) {
            if (tm.getSource() instanceof AbstractWorkModel) {
                buffer.append(tm.getSource().getId()).append(",");
            }
            findForkNodeNames(tm.getSource(), buffer);
        }
    }

    /**
     * 对join节点的所有输入变迁进行递归，查找join至fork节点的所有中间task元素
     *
     * @see com.bocloud.fms.engine.handlers.impl.AbstractMergeHandler#findActiveNodes()
     */
    @Override
    protected String[] findActiveNodes() {
        StringBuilder buffer = new StringBuilder(20);
        findForkNodeNames(model, buffer);
        String[] nodeNames = buffer.toString().split(",");
        return nodeNames;
    }

    @Override
    protected Boolean isOkToMerge(Execution execution) {
        /**
         * 查看join节点相连的几个节点是否符合条件
         */
        Map<String, TransitionModel> nodeNameAndTransition = new HashMap<>(16);
        StringBuilder buffer = new StringBuilder(20);
        List<TransitionModel> inputs = model.getInputs();
        for (TransitionModel tm : inputs) {
            if (tm.getSource() instanceof AbstractWorkModel) {
                String name = tm.getSource().getId();
                buffer.append(name).append(",");
                nodeNameAndTransition.put(name, tm);
            }
        }
        String[] nodeNames = buffer.toString().split(",");
        FlowQueryService queryService = execution.getEngine().query();
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("nodeId", JSONObject.toJSONString(nodeNames).substring(1, JSONObject.toJSONString(nodeNames).length() - 1)), Sign.IN));
        params.add(new Param(MapTools.simpleMap("t.instanceId", execution.getFlowInstance().getId()), Sign.EQ));
        List<FlowNode> nodes = queryService.getNodes(execution.getContext(), new Pager(1, Integer.MAX_VALUE, params, null));
        for (FlowNode node : nodes) {
            TransitionModel tm = nodeNameAndTransition.get(node.getNodeId());
            // 如果节点是活动中，不能合并
            if (FlowConstants.STATE_ACTIVE.equals(node.getNodeState())) {
                return false;
            }
            Map<String, Object> variableMap = node.getVariableMap();
            // 如果审核节点的flag与后面线的条件不一致，不能合并
            String auditFlag = (String) variableMap.get(FlowConstants.ARGS_AUDIT_FLAG);
            if (!auditFlag.equals(tm.getAuditFlag())) {
                return false;
            }
        }

        return true;
    }
}
