package com.bocloud.fms.engine.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.ListTool;
import com.bocloud.fms.common.enums.FlowConstants;
import com.bocloud.fms.common.model.BasicDomainModel;
import com.bocloud.fms.engine.FlowInterceptor;
import com.bocloud.fms.engine.GenericAssignment;
import com.bocloud.fms.engine.freemarker.FreeMarkerUtil;
import com.bocloud.fms.engine.model.AuditTaskModel;
import com.bocloud.fms.engine.model.BaseNodeModel;
import com.bocloud.fms.engine.model.DefinitionModel;
import com.bocloud.fms.engine.model.FlowNodeModel;
import com.bocloud.fms.entity.FlowInstance;
import com.bocloud.fms.entity.FlowNode;
import com.bocloud.fms.intf.FlowQueryService;
import com.bocloud.fms.model.NotifyMessage;
import com.bocloud.fms.service.Execution;
import com.bocloud.fms.service.internal.SmsRoleService;
import com.bocloud.fms.service.internal.SmsTenantService;
import com.bocloud.fms.service.internal.SmsUserService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Set;

/**
 * @author hufy
 */
@Component("auditTaskInterceptor")
public class AuditTaskInterceptor implements FlowInterceptor {

    Logger logger = LoggerFactory.getLogger(this.getClass().getName());
    @Autowired
    FlowQueryService flowQueryService;
    @Autowired
    SmsUserService smsUserService;
    @Autowired
    SmsTenantService smsTenantService;
    @Autowired
    SmsRoleService smsRoleService;
    @Autowired
    GenericAssignment genericAssignment;
    @Autowired
    AmqpTemplate amqpTemplate;

    @Override
    public void intercept(Execution execution) {
        if (ListTool.isEmpty(execution.getTasks())) {
            return;
        }
        try {
            FlowInstance flowInstance = null;
            // 站内信审核操作
            flowInstance = execution.getFlowInstance();
            List<BasicDomainModel<FlowNode, FlowNodeModel>> tasks = execution.getTasks();
            for (BasicDomainModel<FlowNode, FlowNodeModel> task : tasks) {
                if (task != null) {
                    BaseNodeModel model = task.getModel();
                    if (model == null) {
                        DefinitionModel definitionModel = execution.getModel();
                        model = definitionModel.getNode(task.getEntity().getNodeId());
                    }
                    if (model != null && model instanceof AuditTaskModel
                            && !isNotified(execution.getNotifiedNodes(), model)) {
                        String variable = flowInstance.getVariable();
                        JSONObject param = JSONObject.parseObject(variable);
                        String title = param.getString(FlowConstants.APPLY_THEME);
                        FlowNode flowNode = task.getEntity();
                        Set<RequestContext> assignees = genericAssignment.getAssignees((FlowNodeModel) model, flowInstance.getProjectId(), execution.getContext());
                        String content;
                        String template = ((FlowNodeModel) model).getMsgTemplateReceive();
                        if (StringUtils.isBlank(template)) {
                            return;
                        }
                        // ApplyParams转换成json格式，并且将ApplyParams里的参数提到外面一个层级，为了方便生成消息模板
                        if (param.getJSONObject(FlowConstants.APPLY_PARAMS) != null) {
                            param.putAll(param.getJSONObject(FlowConstants.APPLY_PARAMS));
                        }
                        param.put("currentNode", flowNode.getName());
                        param.put("remark", flowNode.getVariableMap().get("remark"));
                        try {
                            content = FreeMarkerUtil.getContent(template, param);
                        } catch (Exception e) {
                            logger.error("生成消息通知内容失败", e);
                            return;
                        }
                        String applyCategory = param.getString(FlowConstants.APPLY_CATEGORY);
                        // F5申请内容信息
                        if (FlowConstants.APPLY_CATEGORY_F5_APPLY.equals(applyCategory)) {
                            JSONObject applyParams = param.getJSONObject(FlowConstants.APPLY_PARAMS);
                            JSONArray members = applyParams.getJSONArray("members");
                            content += "\n";
                            content += "Pool Member:\n";
                            for (int i = 0; i < members.size(); i++) {
                                JSONObject member = members.getJSONObject(i);
                                content += member.getString("poolMemberAddress") + ":"
                                        + member.getString("poolMemberPort");
                                if (i != member.size() - 1) {
                                    content += "\n";
                                }
                            }
                        } else if (FlowConstants.APPLY_CATEGORY_F5_EXPANSION_APPLY.equals(applyCategory)) { // F5扩容申请内容信息
                            JSONObject applyParams = param.getJSONObject(FlowConstants.APPLY_PARAMS);
                            JSONArray members = applyParams.getJSONArray("members");
                            content += "\n";
                            content += "VirtualServers:"
                                    + applyParams.getJSONObject("virtualServer").getString("name") + "\n";
                            content += "新增Pool Member:\n";
                            for (int i = 0; i < members.size(); i++) {
                                JSONObject member = members.getJSONObject(i);
                                content += member.getString("poolMemberAddress") + ":"
                                        + member.getString("poolMemberPort");
                                if (i != member.size() - 1) {
                                    content += "\n";
                                }
                            }
                        }
                        for (RequestContext context : assignees) {
                            NotifyMessage message = new NotifyMessage(context,
                                    "【" + flowInstance.getInstanceNo() + "】《" + title + "》审批通知", content, "<pre>" + content + "</pre>", true, false);
                            amqpTemplate.convertAndSend("event.notice.queue", message);
                        }
                        execution.getNotifiedNodes().add(new BasicDomainModel<>(flowNode, (FlowNodeModel) model));
                    }
                }
            }
        } catch (Exception e) {
            logger.error("拦截器处理失败", e);
        }
    }

    public boolean isNotified(List<BasicDomainModel<FlowNode, FlowNodeModel>> notifiedList, BaseNodeModel toDetect) {
        if (notifiedList.size() == 0) {
            return false;
        }
        for (BasicDomainModel<FlowNode, FlowNodeModel> node : notifiedList) {
            if (node.getModel().getId().equals(toDetect.getId())) {
                return true;
            }
        }
        return false;
    }
}
