package com.bocloud.fms.engine.model;

import com.bocloud.fms.common.enums.FlowConstants;
import com.bocloud.fms.common.utils.StringUtil;
import com.bocloud.fms.service.Execution;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;

/**
 * @author hufy 审核节点task元素
 * @since 1.0
 */
public class AuditTaskModel extends FlowNodeModel {

    private static final Logger log = LoggerFactory.getLogger(AuditTaskModel.class);

    /**
     *
     */
    private static final long serialVersionUID = 1775545243233990922L;

    @Override
    protected void exec(Execution execution) {
        log.info(execution.getFlowInstance().getId() + "->decision execution.getArgs():" + execution.getArgs());
        Map<String, Object> args = execution.getArgs();

        boolean isfound = false;
        for (TransitionModel tm : getOutputs()) {
            // 开始节点出来的直接跳过
            if (tm.getSource().getInputs().size() == 1 && tm.getSource() instanceof StartModel) {
                isfound = true;
            }
            String auditFlag = tm.getAuditFlag();
            if (StringUtil.isNotEmpty(auditFlag)
                    && auditFlag.equalsIgnoreCase((String) args.get(FlowConstants.ARGS_AUDIT_FLAG))) {
                boolean result = true;
                if (StringUtil.isNotEmpty(tm.getExpression())) {
                    result = expression.eval(Boolean.class, tm.getExpression(), args);
                }
                if (result) {
                    tm.setEnabled(true);
                    tm.execute(execution);
                    isfound = true;
                }
            }
        }
        if (!isfound) {
            // 审核未通过的处理,将流程终结
            if (FlowConstants.ARGS_AUDIT_FLAG_NO_PASS.equals(execution.getArgs().get(FlowConstants.ARGS_AUDIT_FLAG))) {
                execution.getEngine().flowInstance().terminate(execution.getFlowInstance().getId(),
                        (String) execution.getArgs().get("FAILED_REASON"));
                return;
            }
            log.error("节点无法确定下一步执行路线，流程图里是不是没有配置该流程下一步的处理，流程id====" + execution.getFlowInstance().getId());
        }
    }

}
