package com.bocloud.fms.engine.model;

import com.alibaba.fastjson.annotation.JSONField;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 流程定义definition元素
 *
 * @author hufy
 */
public class DefinitionModel implements Serializable {
    /**
     *
     */
    private static final long serialVersionUID = -9000210138346445915L;
    /**
     * 节点元素集合
     */
    private List<BaseNodeModel> nodes = new ArrayList<BaseNodeModel>();
    private List<FlowNodeModel> flowNodeModels = new ArrayList<FlowNodeModel>();
    /**
     * 流程实例启动url
     */
    private String instanceUrl;
    /**
     * 期望完成时间
     */
    private Date expireTime;
    /**
     * lock
     */
    private final Object lock = new Object();

    /**
     * 获取所有的有序任务模型集合
     *
     * @return List<FlowNodeModel> 任务模型集合
     */
    @JSONField(serialize = false)
    public List<FlowNodeModel> getFlowNodeModels() {
        if (flowNodeModels.isEmpty()) {
            synchronized (lock) {
                if (flowNodeModels.isEmpty()) {
                    buildModels(flowNodeModels, getStart().getNextModels(FlowNodeModel.class), FlowNodeModel.class);
                }
            }
        }
        return flowNodeModels;
    }

    /**
     * 根据指定的节点类型返回流程定义中所有模型对象
     *
     * @param clazz 节点类型
     * @param <T>   泛型
     * @return 节点列表
     */
    public <T> List<T> getModels(Class<T> clazz) {
        List<T> models = new ArrayList<T>();
        buildModels(models, getStart().getNextModels(clazz), clazz);
        return models;
    }

    private <T> void buildModels(List<T> models, List<T> nextModels, Class<T> clazz) {
        for (T nextModel : nextModels) {
            if (!models.contains(nextModel)) {
                models.add(nextModel);
                buildModels(models, ((BaseNodeModel) nextModel).getNextModels(clazz), clazz);
            }
        }
    }

    /**
     * 获取definition定义的start节点模型
     *
     * @return
     */
    public StartModel getStart() {
        for (BaseNodeModel node : nodes) {
            if (node instanceof StartModel) {
                return (StartModel) node;
            }
        }
        return null;
    }

    /**
     * 获取definition定义的指定节点名称的节点模型
     *
     * @param nodeName 节点名称
     * @return
     */
    public BaseNodeModel getNode(String nodeName) {
        for (BaseNodeModel node : nodes) {
            if (node.getId().equals(nodeName)) {
                return node;
            }
        }
        return null;
    }

    /**
     * 判断当前模型的节点是否包含给定的节点名称参数
     *
     * @param nodeNames 节点名称数组
     * @return
     */
    public <T> boolean containsNodeNames(Class<T> tClass, String... nodeNames) {
        for (BaseNodeModel node : nodes) {
            if (!tClass.isInstance(node)) {
                continue;
            }
            for (String nodeName : nodeNames) {
                if (node.getId().equals(nodeName)) {
                    return true;
                }
            }
        }
        return false;
    }

    public List<BaseNodeModel> getNodes() {
        return nodes;
    }

    public void setNodes(List<BaseNodeModel> nodes) {
        this.nodes = nodes;
    }

    public Date getExpireTime() {
        return expireTime;
    }

    public void setExpireTime(Date expireTime) {
        this.expireTime = expireTime;
    }

    public String getInstanceUrl() {
        return instanceUrl;
    }

    public void setInstanceUrl(String instanceUrl) {
        this.instanceUrl = instanceUrl;
    }

}
