package com.bocloud.fms.engine.model;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.RequestContext;
import com.bocloud.fms.common.enums.FlowConstants;
import com.bocloud.fms.common.model.BasicDomainModel;
import com.bocloud.fms.common.utils.AbstractAssertUtil;
import com.bocloud.fms.entity.FlowInstance;
import com.bocloud.fms.entity.FlowNode;
import com.bocloud.fms.service.Execution;

import java.util.*;

/**
 * 开始节点定义start元素
 *
 * @author hufy
 */
public class StartModel extends FlowNodeModel {

    @Override
    public String getCode() {
        return "DRAFT";
    }

    /**
     *
     */
    private static final long serialVersionUID = -4550530562581330477L;

    /**
     * 开始节点无输入变迁
     */
    @Override
    public List<TransitionModel> getInputs() {
        return Collections.emptyList();
    }

    @Override
    protected void exec(Execution execution) {
        // isDraft:暂存
        boolean isDraft = execution.getArgs().get(FlowConstants.ARGS_IS_DRAFT) != null
                && Boolean.valueOf(String.valueOf(execution.getArgs().get(FlowConstants.ARGS_IS_DRAFT)));
        // 首次开启流程
        if (execution.getTask() == null) {
            List<BasicDomainModel<FlowNode, FlowNodeModel>> tasks = createTask(this, execution, isDraft);

            if (tasks != null && tasks.size() > 0) {
                FlowInstance flowInstance = execution.getEngine().query()
                        .getFlowInstance(execution.getFlowInstance().getId());
                AbstractAssertUtil.notNull(flowInstance,
                        "指定的流程实例[id=" + execution.getFlowInstance().getId() + "]已完成或不存在");
                flowInstance.setGmtUpdate(new Date());
                execution.getEngine().flowInstance().updateFlowInstance(flowInstance);
            }
        }
        if (!isDraft) {
            runOutTransition(execution);
        }
    }

    public List<BasicDomainModel<FlowNode, FlowNodeModel>> createTask(StartModel model, Execution execution, boolean isDraft) {
        List<BasicDomainModel<FlowNode, FlowNodeModel>> tasks = new ArrayList<>();
        Map<String, Object> args = execution.getArgs();
        if (args == null) {
            args = new HashMap<>(20);
        }
        args.put(FlowNode.KEY_ACTOR, JSONObject.toJSONString(Collections.singleton(execution.getContext())));
        // 存入数据库
        BasicDomainModel<FlowNode, FlowNodeModel> domain = new BasicDomainModel<FlowNode, FlowNodeModel>(new FlowNode(),
                model);
        domain.getEntity().setInstanceId(execution.getFlowInstance().getId());
        domain.getEntity().setNodeId(model.getId());
        domain.getEntity().setName(model.getName());
        domain.getEntity().setGmtCreate(new Date());
        if (!isDraft) {
            domain.getEntity().setGmtFinish(new Date());
            domain.getEntity().setOperatorId(execution.getFlowInstance().getCreatorId());
            domain.getEntity().setOperatorCatalog(execution.getFlowInstance().getCreatorCatalog() == RequestContext.Catalog.Tenant ? RequestContext.Catalog.Tenant : RequestContext.Catalog.User);
        }
        domain.getEntity().setVersion(execution.getFlowInstance().getVersion());
        domain.getEntity().setNodeState(isDraft ? FlowConstants.STATE_ACTIVE : FlowConstants.STATE_FINISH);
        domain.getEntity().setCode(model.getCode());
        domain.getEntity().setParentNodeId(-1L);
        domain.setModel(model);
        domain.getEntity().setVariable(JSONObject.toJSONString(args));
        FlowNode flowTask = execution.getEngine().node().saveTask(domain.getEntity(), Collections.singletonList(execution.getContext()));
        domain = new BasicDomainModel<>(flowTask);
        execution.setTask(domain);
        tasks.add(domain);
        return tasks;
    }

}
