package com.bocloud.fms.engine.parser;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.fms.engine.model.BaseNodeModel;
import com.bocloud.fms.engine.model.EndModel;
import com.bocloud.fms.engine.model.TransitionModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 抽象dom节点解析类 完成通用的属性、变迁解析
 *
 * @author hufy
 */
public abstract class AbstractNodeParser implements NodeParser {

    Logger logger = LoggerFactory.getLogger(this.getClass().getName());
    /**
     * 模型对象
     */
    protected BaseNodeModel model;

    /**
     * 实现NodeParser接口的parse函数
     * 由子类产生各自的模型对象，设置常用的名称属性，并且解析子节点transition，构造TransitionModel模型对象
     */
    @Override
    public void parse(JSONObject jsonObject) {
        model = newModel();
        model.setId(jsonObject.getString(ATTR_ID));
        model.setName(jsonObject.getString(ATTR_NAME));

        // 解析线节点
        JSONArray transitions = jsonObject.getJSONArray(NODE_TRANSITIONS);
        if (transitions != null && transitions.size() > 0) {
            for (Object te : transitions) {
                JSONObject node = (JSONObject) te;
                TransitionModel transition = new TransitionModel();
                transition.setId(node.getString(ATTR_ID));
                transition.setName(node.getString(ATTR_NAME));
                transition.setTo(node.getString(ATTR_TO));
                transition.setAuditFlag(node.getString(ATTR_AUDITFLAG));
                transition.setSource(model);
                model.getOutputs().add(transition);
            }
        } else {
            if (!(model instanceof EndModel)) {
                logger.warn("没有变迁线的节点" + model.getId());
            }
        }

        // 解析线节点的子节点
        parseNode(model, jsonObject);
    }

    /**
     * 子类可覆盖此方法，完成特定的解析
     *
     * @param model
     * @param node
     */
    protected void parseNode(BaseNodeModel model, JSONObject node) {

    }

    /**
     * 抽象方法，由子类产生各自的模型对象
     *
     * @return
     */
    protected abstract BaseNodeModel newModel();

    /**
     * 返回模型对象
     */
    @Override
    public BaseNodeModel getModel() {
        return model;
    }
}
