package com.bocloud.fms.engine.parser;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.fms.common.exception.FlowException;
import com.bocloud.fms.engine.model.BaseNodeModel;
import com.bocloud.fms.engine.model.DefinitionModel;
import com.bocloud.fms.engine.model.TransitionModel;
import com.bocloud.fms.service.AbstractServiceContext;

/**
 * 流程定义xml文件的模型解析器
 *
 * @author hufy
 */
public class ModelParser {

    /**
     * 解析流程定义文件，并将解析后的对象放入模型容器中
     *
     * @param graph
     */
    public static DefinitionModel parse(String graph) {
        JSONArray nodes = (JSONArray) JSONObject.parseArray(graph);
        DefinitionModel definition = new DefinitionModel();
        for (Object node : nodes) {
            /**
             * 将节点解析为对应的模型
             */
            BaseNodeModel model = parseModel((JSONObject) node);
            definition.getNodes().add(model);
        }

        // 循环节点模型，构造变迁输入、输出的source、target ，连接结构
        for (BaseNodeModel node : definition.getNodes()) {
            for (TransitionModel transition : node.getOutputs()) {
                String to = transition.getTo();
                for (BaseNodeModel node2 : definition.getNodes()) {
                    if (to.equalsIgnoreCase(node2.getId())) {
                        node2.getInputs().add(transition);
                        transition.setTarget(node2);
                    }
                }
            }
        }
        return definition;
    }

    /**
     * 对流程定义xml的节点，根据其节点对应的解析器解析节点内容
     *
     * @param node
     * @return
     */
    private static BaseNodeModel parseModel(JSONObject node) {
        String nodeType = node.getString(NodeParser.ATTR_TYPE);
        NodeParser nodeParser = null;
        try {
            /**
             * 根据对应节点名称获取对应的节点解析类
             */
            nodeParser = AbstractServiceContext.getContext().findByName(nodeType + "NodeParser", NodeParser.class);
            nodeParser.parse(node);
            return nodeParser.getModel();
        } catch (RuntimeException e) {
            throw new FlowException(e);
        }
    }

}
