package com.bocloud.fms.intf;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.fms.common.model.BasicDomainModel;
import com.bocloud.fms.engine.model.DefinitionModel;
import com.bocloud.fms.entity.FlowDefinition;
import com.bocloud.fms.model.DefinitionParams;

import java.util.List;

/**
 * 流程定义业务类
 *
 * @author hufy
 */
public interface FlowDefinitionService {

    void check(FlowDefinition definition, Long id);

    /**
     * 保存流程定义
     *
     * @param definition 流程定义对象
     */
    void saveDefinition(BasicDomainModel<FlowDefinition, DefinitionModel> definition);

    /**
     * 根據InputStream輸入流，部署流程定义
     *
     * @param definitionParams 流程定义输入流
     * @param context          创建人
     * @return String 流程定义id
     */
    Long deploy(DefinitionParams definitionParams, RequestContext context);

    /**
     * 根據InputStream輸入流，部署流程定义
     *
     * @param id               流程定义id
     * @param definitionParams 输入的模板
     * @param context          用户
     * @return 返回执行结果
     */
    BocloudResult redeploy(Long id, DefinitionParams definitionParams, RequestContext context);

    /**
     * 卸载指定的流程定义，只更新状态
     *
     * @param id      流程定义id
     * @param context
     * @param remark  终止的原因
     * @return 返回终止的结果
     */
    boolean terminate(Long id, RequestContext context, String remark);

    /**
     * 查询最新的版本号
     *
     * @param id 流程定义id
     * @return 返回最新的版本号
     */
    Integer getLatestVersion(Long id);

    /**
     * 获取最新的流程定义
     *
     * @param id 流程定义id
     * @return 返回最新的流程定义
     */
    BasicDomainModel<FlowDefinition, DefinitionModel> getLatestDefinition(Long id);

    /**
     * 根据id获取流程定义
     *
     * @param id
     * @return 返回流程定义
     */
    BasicDomainModel<FlowDefinition, DefinitionModel> getDefinition(Long id);

    /**
     * 根据查询的参数，分页对象，返回分页后的查询结果
     *
     * @param pager  分页对象
     * @param latest 是否查询最新版本
     * @return List<Definition> 流程定义集合
     */
    int count(RequestContext context, Pager pager, boolean latest);

    /**
     * 根据查询的参数，分页对象，返回分页后的查询结果
     *
     * @param pager  分页对象
     * @param latest 是否查询最新版本
     * @return List<Definition> 流程定义集合
     */
    List<FlowDefinition> getDefinitions(RequestContext context, Pager pager, boolean latest);

    /**
     * 查询历史
     *
     * @param context
     * @param id
     * @return 返回定义的历史
     */
    List<FlowDefinition> getDefinitionHistory(RequestContext context, Long id);

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return 返回执行结果
     */
    BocloudResult remove(Long id, RequestContext context);

    /**
     * 判断当前用户可否开启流程
     *
     * @param entity
     * @param tenantId
     */
    void checkPermision(FlowDefinition entity, Long tenantId);

    /**
     * 判断名称是否存在
     *
     * @param name
     * @param tenantId
     * @return
     */
    Boolean isNameExsit(String name, Long tenantId);

    /**
     * 重新激活终止的流程定义
     *
     * @param id
     * @param context
     * @param remark
     * @return
     */
    boolean active(Long id, RequestContext context, String remark);
}
