package com.bocloud.fms.intf;

import com.bocloud.common.model.RequestContext;
import com.bocloud.fms.common.model.BasicDomainModel;
import com.bocloud.fms.engine.model.DefinitionModel;
import com.bocloud.fms.entity.FlowDefinition;
import com.bocloud.fms.entity.FlowInstance;

import java.util.Map;

/**
 * 流程实例业务类
 *
 * @author hufy
 */
public interface FlowInstanceService {

    /**
     * 根据流程、操作人员、父流程实例ID创建流程实例
     *
     * @param definition     流程定义对象
     * @param context        当前用户
     * @param project        项目ID
     * @param args           参数列表
     * @param parentId       父流程实例ID
     * @param parentNodeName 父流程节点模型
     * @return 返回创建的流程实例
     */
    FlowInstance createFlowInstance(BasicDomainModel<FlowDefinition, DefinitionModel> definition,
                                    RequestContext context, Long project, Map<String, Object> args, Long parentId, String parentNodeName);

    /**
     * 流程实例正常完成
     *
     * @param flowInstanceId 流程实例id
     */
    void complete(Long flowInstanceId);

    /**
     * 流程实例强制终止
     *
     * @param instanceId 流程实例id
     * @param reason
     */
    void terminate(Long instanceId, String reason);

    /**
     * 流程实例撤销
     *
     * @param instanceId 流程实例id
     * @param operator   处理人员
     * @param reason     取消原因
     * @param notice     是否通知
     * @return 取消结果
     */
    boolean cancel(Long instanceId, Long operator, String reason, boolean notice);

    /**
     * 更新流程实例
     *
     * @param flowInstance 流程实例对象
     */
    void updateFlowInstance(FlowInstance flowInstance);

    /**
     * 根据流程实例id查询实例对象
     *
     * @param flowInstanceId 活动流程实例id
     * @return FlowInstance 活动流程实例对象
     */
    FlowInstance getFlowInstance(Long flowInstanceId);

}
