/* Copyright 2013-2015 www.snakerflow.com.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.bocloud.fms.intf;

import com.bocloud.common.model.RequestContext;
import com.bocloud.fms.common.model.BasicDomainModel;
import com.bocloud.fms.engine.model.DefinitionModel;
import com.bocloud.fms.engine.model.FlowNodeModel;
import com.bocloud.fms.entity.FlowNode;
import com.bocloud.fms.service.Execution;

import java.util.List;
import java.util.Map;

/**
 * 任务业务类，包括以下服务： 1、创建任务 2、添加、删除参与者 3、完成任务 4、撤回任务 5、回退任务 6、提取任务
 *
 * @author hufy
 */
public interface FlowNodeService {

    /**
     * 按taskId获取
     *
     * @param taskId
     * @return
     */
    FlowNode getFlowNode(Long taskId);

    /**
     * 完成指定的任务，删除活动任务记录，创建历史任务
     *
     * @param taskId  任务id
     * @param context 操作人
     * @return Task 任务对象
     */
    BasicDomainModel<FlowNode, FlowNodeModel> complete(Long taskId, RequestContext context);

    /**
     * 根据任务主键ID，操作人ID完成任务
     *
     * @param taskId   任务id
     * @param operator 操作人id
     * @param args     参数集合
     * @return Task 任务对象
     */
    BasicDomainModel<FlowNode, FlowNodeModel> complete(Long taskId, RequestContext context, Map<String, Object> args);

    /**
     * 更新任务对象
     *
     * @param task 任务对象
     */
    void updateTask(FlowNode task);

    /**
     * 根据历史任务主键id，操作人唤醒历史任务 该方法会导致流程状态不可控，请慎用
     *
     * @param taskId   历史任务id
     * @param operator 操作人id
     * @return Task 唤醒后的任务对象
     */
    FlowNode resume(Long taskId, Long operator);

    /**
     * 根据任务主键id、操作人撤回任务
     *
     * @param taskId   任务id
     * @param operator 操作人
     * @return Task 任务对象
     */
    FlowNode withdrawTask(Long taskId, Long operator);

    /**
     * 根据当前任务对象驳回至上一步处理
     *
     * @param model       流程定义模型，用以获取上一步模型对象
     * @param currentTask 当前任务对象
     * @return Task 任务对象
     */
    BasicDomainModel<FlowNode, FlowNodeModel> rejectNode(DefinitionModel model, FlowNode currentTask);

    /**
     * 根据taskId、operator，判断操作人operator是否允许执行任务
     *
     * @param task     任务对象
     * @param operator 操作人
     * @return boolean 是否允许操作
     */
    boolean isAllowed(FlowNode task, RequestContext operator);

    /**
     * 根据任务模型、执行对象创建新的任务
     *
     * @param flowNodeModel 任务模型
     * @param execution     执行对象
     * @return List<Task> 创建任务集合
     */
    List<BasicDomainModel<FlowNode, FlowNodeModel>> createTask(FlowNodeModel flowNodeModel, Execution execution);

    /**
     * 根据任务id获取任务模型
     *
     * @param taskId 任务id
     * @return
     */
    FlowNodeModel getTaskModel(Long taskId);

    /**
     * 撤回任务,暂时没用
     *
     * @param task 节点id
     * @return
     */
    BasicDomainModel<FlowNode, FlowNodeModel> undoNode(BasicDomainModel<FlowNode, FlowNodeModel> task);

    /**
     * 保存任务
     *
     * @param task
     * @param actors
     * @return
     */
    FlowNode saveTask(FlowNode task, List<RequestContext> actors);
}
