/* Copyright 2013-2015 www.snakerflow.com.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.bocloud.fms.intf;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.fms.common.model.BasicDomainModel;
import com.bocloud.fms.engine.model.FlowNodeModel;
import com.bocloud.fms.entity.FlowInstance;
import com.bocloud.fms.entity.FlowNode;

import java.util.List;

/**
 * 流程相关的查询服务
 *
 * @author hufy
 * @since 1.0
 */
public interface FlowQueryService {
    /**
     * st<FlowDefinition> getDefinitio 根据流程实例ID获取流程实例对象
     *
     * @param flowInstanceId 流程实例id
     * @return flowInstance 流程实例对象
     */
    FlowInstance getFlowInstance(Long flowInstanceId);

    /**
     * 根据条件查询任务
     *
     * @param context
     * @param pager
     * @return
     */
    List<FlowNode> getNodes(RequestContext context, Pager pager);

    /**
     * 获取活动节点
     *
     * @param params 过滤条件
     * @return 返回的活动状态的流程节点
     */
    List<BasicDomainModel<FlowNode, FlowNodeModel>> getActiveNodes(List<Param> params);

    /**
     * 分页查询流程实例
     *
     * @param pager   分页条件
     * @param context 当前登录用户
     * @param type    类型
     * @return
     */
    BocloudResult getGridBeanOfFlowInstances(Pager pager, RequestContext context, String type);

    /**
     * 查询用户显示名称
     *
     * @param context
     * @return
     */
    String getDisplayName(RequestContext context);

    /**
     * 获取当前审核人
     *
     * @param flowInstance
     * @param context
     * @return
     */
    String getCurrentAuditors(FlowInstance flowInstance, RequestContext context);

    /**
     * 待审批工单数量
     *
     * @param context
     * @return
     */
    long countTodo(RequestContext context);
}
