/* Copyright 2013-2015 www.snakerflow.com.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.bocloud.fms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.fms.common.enums.FlowConstants;
import com.bocloud.fms.common.model.BasicDomainModel;
import com.bocloud.fms.engine.Completion;
import com.bocloud.fms.engine.InstanceNoGenerator;
import com.bocloud.fms.engine.model.DefinitionModel;
import com.bocloud.fms.entity.FlowDefinition;
import com.bocloud.fms.entity.FlowInstance;
import com.bocloud.fms.intf.FlowInstanceService;
import com.bocloud.fms.intf.FlowQueryService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

/**
 * @author hufy
 */
@Service
public class FlowInstanceServiceImpl extends AbstractAccessService implements FlowInstanceService {

    private Logger logger = LoggerFactory.getLogger(this.getClass().getName());
    @Autowired
    AmqpTemplate amqpTemplate;
    @Autowired
    FlowQueryService flowQueryService;
    /**
     * 工单编号生成器
     */
    @Autowired
    InstanceNoGenerator instanceNoGenerator;

    @Override
    public FlowInstance createFlowInstance(BasicDomainModel<FlowDefinition, DefinitionModel> definitionDomain,
                                           RequestContext context, Long project, Map<String, Object> args, Long parentId, String parentNodeName) {
        FlowInstance flowInstance = new FlowInstance();
        flowInstance.setParentId(parentId);
        flowInstance.setParentNodeName(parentNodeName);
        flowInstance.setGmtCreate(new Date());
        flowInstance.setGmtUpdate(flowInstance.getGmtCreate());
        flowInstance.setCreatorId(context.getTarget());
        flowInstance.setCreatorCatalog(context.getCatalog());
        flowInstance.setMenderId(flowInstance.getCreatorId());
        flowInstance.setMenderCatalog(context.getCatalog());
        flowInstance.setDefId(definitionDomain.getEntity().getId());
        flowInstance.setInstanceState(FlowConstants.STATE_ACTIVE);
        flowInstance.setVersion(definitionDomain.getEntity().getVersion());
        flowInstance.setName(definitionDomain.getEntity().getName());
        flowInstance.setProjectId(project);
        if (null != args && args.get(FlowConstants.APPLY_THEME) != null) {
            flowInstance.setName((String) args.get(FlowConstants.APPLY_THEME));
        }
        String instanceNo = null;
        if (null != args && null != args.get(FlowConstants.ARGS_INSTANCE_NO)) {
            instanceNo = String.valueOf(args.get(FlowConstants.ARGS_INSTANCE_NO));
        } else {
            instanceNo = instanceNoGenerator.generate();
        }
        flowInstance.setInstanceNo(instanceNo);
        if (null != args) {
            if (args.get(FlowConstants.APPLY_THEME) != null) {
                flowInstance.setName((String) args.get(FlowConstants.APPLY_THEME));
            } else {
                args.put(FlowConstants.APPLY_THEME, flowInstance.getName());
            }
            if (args.get(FlowConstants.ARGS_PROJECT_ID) != null) {
                flowInstance.setProjectId(Long.valueOf(String.valueOf(args.get(FlowConstants.ARGS_PROJECT_ID))));
            }
            args.put(FlowConstants.ARGS_INSTANCE_NO, flowInstance.getInstanceNo());
            String creatorName = flowQueryService.getDisplayName(new RequestContext(flowInstance.getCreatorCatalog(), flowInstance.getCreatorId(), null, null, null));
            args.put("creatorName", creatorName);
            flowInstance.setApplyCategory(String.valueOf(args.get(FlowConstants.APPLY_CATEGORY)));
            flowInstance.setVariable(JSONObject.toJSONString(args));
        }
        this.getFlowInstanceRepository().save(flowInstance);
        flowInstance.getVariableMap().put(Common.ID, flowInstance.getId());
        flowInstance.setVariable(JSONObject.toJSONString(flowInstance.getVariableMap()));
        this.getFlowInstanceRepository().update(flowInstance);
        return flowInstance;
    }

    @Override
    public void terminate(Long instanceId, String reason) {
        FlowInstance flowInstance = this.getFlowInstanceRepository().query(instanceId);
        flowInstance.setInstanceState(FlowConstants.STATE_TERMINATION);
        Date currentDate = new Date();
        flowInstance.setGmtUpdate(currentDate);
        // 设置完成时间
        flowInstance.setGmtFinish(currentDate);
        flowInstance.setRemark(reason);
        Boolean isSuccess = this.getFlowInstanceRepository().update(flowInstance);
        if (isSuccess) {
            logger.info("发送终止消息");
            sendTeminateMsg(flowInstance,false);
        }
    }

    private void sendTeminateMsg(FlowInstance flowInstance,Boolean isCancel) {
        String variable = flowInstance.getVariable();
        JSONObject paramJSON = JSONObject.parseObject(variable);
        String category = paramJSON.getString(FlowConstants.APPLY_CATEGORY);
        paramJSON.put(FlowConstants.APPLY_STATUS, FlowConstants.APPLY_STATUS_UNRATIFIED);
        paramJSON.put(FlowConstants.APPLY_IS_CANCEL, isCancel);
        if (FlowConstants.APPLY_CATEGORY_SERVICE_RELEASE.equals(category)
                || FlowConstants.APPLY_CATEGORY_SERVICE_UNRELEASE.equals(category)
                || FlowConstants.APPLY_CATEGORY_SERVICE_STOP.equals(category)
                || FlowConstants.APPLY_CATEGORY_SERVICE_APPLY.equals(category)
                || FlowConstants.APPLY_CATEGORY_SERVICE_POSTPONE.equals(category)) {
            amqpTemplate.convertAndSend(FlowConstants.SERVICE_APPLY_QUEUE, paramJSON);
        } else if (FlowConstants.APPLY_CATEGORY_COMMON_TASK_START.equals(category)
                || FlowConstants.APPLY_CATEGORY_SIMPLE_TASK_START.equals(category)
                || FlowConstants.APPLY_CATEGOYR_CRON_TASK_START.equals(category)
                || FlowConstants.APPLY_CATEGORY_SCRIPT_UPLOAD.equals(category)
                || FlowConstants.APPLY_CATEGOYR_SOFTWARE_UPLOAD.equals(category)
                || FlowConstants.APPLY_CATEGORY_RESOURCE_COMPUTER_APPLY.equals(category)
                || FlowConstants.APPLY_CATEGORY_RESOURCE_IMAGE_APPLY.equals(category)
                || FlowConstants.APPLY_CATEOGRY_RESOURCE_STORAGE_APPLY.equals(category)
                || FlowConstants.APPLY_CATEGORY_RESOURCE_SNAPSHOT_APPLY.equals(category)
                || FlowConstants.APPLY_CATEGORY_RESOURCE_ROUTE_APPLY.equals(category)
                || FlowConstants.APPLY_CATEGORY_RESOURCE_FLOATING_IP_APPLY.equals(category)) {
            amqpTemplate.convertAndSend(FlowConstants.BOCLOUD_FLOW_QUEUE, paramJSON);
        } else if (FlowConstants.APPLY_CATEGORY_RESOURCE_SHOPPING_APPLY.equals(category)) {
            JSONArray jsonArray = paramJSON.getJSONArray(FlowConstants.APPLY_PARAMS);
            for (JSONObject jsonObject : jsonArray.toJavaList(JSONObject.class)) {
                jsonObject.put(FlowConstants.APPLY_STATUS, FlowConstants.APPLY_STATUS_UNRATIFIED);
                amqpTemplate.convertAndSend(FlowConstants.BOCLOUD_FLOW_QUEUE, paramJSON);
            }
        } else {
            amqpTemplate.convertAndSend(FlowConstants.BOCLOUD_FLOW_QUEUE, paramJSON);
        }
    }

    @Override
    public boolean cancel(Long instanceId, Long operator, String reason, boolean notice) {
        FlowInstance flowInstance = this.getFlowInstanceRepository().query(instanceId);
        // 只有活动中的流程才可以撤销
        if (FlowConstants.STATE_ACTIVE.equals(flowInstance.getInstanceState())) {
            if (flowInstance.getCreatorId().equals(operator)) {
                flowInstance.setInstanceState(FlowConstants.STATE_CANCEL);
                Date currentDate = new Date();
                flowInstance.setGmtUpdate(currentDate);
                flowInstance.setGmtFinish(currentDate);
                flowInstance.setRemark(reason);
                boolean isSuccess = this.getFlowInstanceRepository().update(flowInstance);
                if (isSuccess && notice) {
                    sendTeminateMsg(flowInstance,true);
                }
                return isSuccess;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    @Override
    public void updateFlowInstance(FlowInstance flowInstance) {
        this.getFlowInstanceRepository().update(flowInstance);
    }

    @Override
    public void complete(Long flowInstanceId) {
        FlowInstance flowInstance = this.getFlowInstanceRepository().query(flowInstanceId);
        flowInstance.setInstanceState(FlowConstants.STATE_FINISH);
        flowInstance.setGmtFinish(new Date());
        this.getFlowInstanceRepository().update(flowInstance);
        Completion completion = getCompletion();
        if (completion != null) {
            completion.complete(flowInstance);
        }
    }

    @Override
    public FlowInstance getFlowInstance(Long flowInstanceId) {
        return getFlowInstanceRepository().query(flowInstanceId);
    }

}