package com.bocloud.fms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.fms.common.enums.FlowConstants;
import com.bocloud.fms.common.model.BasicDomainModel;
import com.bocloud.fms.common.utils.AbstractAssertUtil;
import com.bocloud.fms.engine.model.BaseNodeModel;
import com.bocloud.fms.engine.model.DefinitionModel;
import com.bocloud.fms.engine.model.FlowNodeModel;
import com.bocloud.fms.entity.FlowDefinition;
import com.bocloud.fms.entity.FlowInstance;
import com.bocloud.fms.entity.FlowNode;
import com.bocloud.fms.intf.FlowQueryService;
import com.bocloud.fms.internel.entity.Role;
import com.bocloud.fms.internel.entity.Tenant;
import com.bocloud.fms.internel.entity.User;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 查询服务实现类
 *
 * @author hufy
 */
@Slf4j
@Service
public class QueryServiceImpl extends AbstractAccessService implements FlowQueryService {

    @Override
    public FlowInstance getFlowInstance(Long flowInstanceId) {
        FlowInstance flowInstance = this.getFlowInstanceRepository().query(flowInstanceId);
        flowInstance.setCreatorName(getDisplayName(
                new RequestContext(flowInstance.getCreatorCatalog(), flowInstance.getCreatorId(), null, null, null)));
        return flowInstance;
    }

    @Override
    public List<FlowNode> getNodes(RequestContext context, Pager pager) {
        List<FlowNode> nodes = this.getFlowNodeRepository().getNodes(pager);
        for (FlowNode flowNode : nodes) {
            if (null == flowNode.getOperatorId()) {
                continue;
            }
            flowNode.setOperatorName(getDisplayName(
                    new RequestContext(flowNode.getOperatorCatalog(), flowNode.getOperatorId(), null, null, null)));
        }
        return nodes;
    }

    @Override
    public List<BasicDomainModel<FlowNode, FlowNodeModel>> getActiveNodes(List<Param> params) {
        Map<String, String> sorter = new LinkedHashMap<>();
        sorter.put("gmtCreate", Common.ONE);
        sorter.put("gmtFinish", Common.ONE);
        sorter.put(Common.ID, Common.ONE);
        List<FlowNode> activeTasks = this.getFlowNodeRepository()
                .getNodes(new Pager(1, Integer.MAX_VALUE, params, sorter));
        AbstractAssertUtil.notNull(activeTasks);
        List<BasicDomainModel<FlowNode, FlowNodeModel>> domainList = new ArrayList<>();
        for (FlowNode activeTask : activeTasks) {
            domainList.add(new BasicDomainModel<>(activeTask));
        }
        return domainList;
    }

    @Override
    public BocloudResult getGridBeanOfFlowInstances(Pager pager, RequestContext context, String type) {
        switch (type) {
            case "SUBMIT":
                pager.getParams().add(new Param(MapTools.simpleMap("o.creatorId", context.getTarget()), Sign.EQ));
                pager.getParams().add(new Param(MapTools.simpleMap("o.creatorCatalog", RequestContext.Catalog.Tenant == context.getCatalog() ? RequestContext.Catalog.Tenant : RequestContext.Catalog.User), Sign.EQ));
                pager.setSorter(Collections.singletonMap("o.gmtCreate", Common.ONE));
                break;
            case "TOAUDIT":
                pager.getParams().add(new Param(MapTools.simpleMap("actorId", context.getTarget()), Sign.EQ));
                pager.getParams().add(new Param(MapTools.simpleMap("actorCatalog", RequestContext.Catalog.Tenant == context.getCatalog() ? RequestContext.Catalog.Tenant : RequestContext.Catalog.User), Sign.EQ));
                pager.getParams().add(new Param(MapTools.simpleMap("instanceState", FlowConstants.STATE_ACTIVE), Sign.EQ));
                pager.getParams().add(new Param(MapTools.simpleMap("nodeState", FlowConstants.STATE_ACTIVE), Sign.EQ));
//                pager.getParams().add(new Param(MapTools.simpleMap("o.tenantId", context.getTenant()), Sign.EQ));
                pager.setSorter(Collections.singletonMap("o.gmtUpdate", Common.ONE));
                break;
            case "AUDITED":
                pager.getParams().add(new Param(MapTools.simpleMap("operatorId", context.getTarget()), Sign.EQ));
                pager.getParams().add(new Param(MapTools.simpleMap("operatorCatalog", RequestContext.Catalog.Tenant == context.getCatalog() ? RequestContext.Catalog.Tenant : RequestContext.Catalog.User), Sign.EQ));
                pager.getParams().add(new Param(MapTools.simpleMap("nodeState", FlowConstants.STATE_FINISH), Sign.EQ));
                pager.setSorter(Collections.singletonMap("o.gmtUpdate", Common.ONE));
                break;
            case "ALL":
                log.info("get flow Instances All");
                pager.setSorter(Collections.singletonMap("o.gmtCreate", Common.ONE));
                break;
            default:
                return new BocloudResult(false, "参数错误");
        }
        Long count = this.getFlowInstanceRepository().count(pager);
        if (0L == count) {
            return new BocloudResult(true,
                    GridHelper.getBean(pager.getPage(), pager.getRows(), count.intValue(), new ArrayList<>()), "");
        }
        List<FlowInstance> flowInstances = this.getFlowInstanceRepository().getFlowInstances(pager);

        if (ListTool.isEmpty(flowInstances)) {
            return new BocloudResult(true,
                    GridHelper.getBean(pager.getPage(), pager.getRows(), count.intValue(), new ArrayList<>()), "");
        }
        for (FlowInstance flowInstance : flowInstances) {
            flowInstance.setCreatorName(this.getDisplayName(new RequestContext(flowInstance.getCreatorCatalog(), flowInstance.getCreatorId(), null, null, null)));
            flowInstance.setMenderName(this.getDisplayName(new RequestContext(flowInstance.getMenderCatalog(), flowInstance.getMenderId(), null, null, null)));
        }

        // 处理等待时长字段，用于在待审批列表中显示
        if ("TOAUDIT".equals(type)) {
            wrapDuration(flowInstances, context);
        }
        // 设置当前审批人信息
        for (FlowInstance flowInstance : flowInstances) {
            if (FlowConstants.STATE_ACTIVE.equals(flowInstance.getInstanceState())) {
                String currentAuditors = getCurrentAuditors(flowInstance, context);
                flowInstance.setCurrentAuditors(currentAuditors);
            }
        }
        return new BocloudResult(true,
                GridHelper.getBean(pager.getPage(), pager.getRows(), count.intValue(), flowInstances), "");
    }

    /**
     * 计算等待时长
     *
     * @param flowInstances
     * @param context
     */
    private void wrapDuration(List<FlowInstance> flowInstances, RequestContext context) {
        Set<Long> instanceIds = new HashSet<>();
        for (FlowInstance flowInstance : flowInstances) {
            instanceIds.add(flowInstance.getId());
        }
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("t.instanceId", JSONObject.toJSONString(instanceIds).substring(1, JSONObject.toJSONString(instanceIds).length() - 1)), Sign.IN));
        params.add(new Param(MapTools.simpleMap("actorId", context.getTarget()), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("actorCatalog", RequestContext.Catalog.Tenant == context.getCatalog() ? RequestContext.Catalog.Tenant : RequestContext.Catalog.User), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("nodeState", FlowConstants.STATE_ACTIVE), Sign.EQ));

        List<FlowNode> nodes = this.flowNodeRepository.getNodes(new Pager(1, Integer.MAX_VALUE, params, null));
        for (FlowInstance flowInstance : flowInstances) {
            for (FlowNode node : nodes) {
                if (flowInstance.getId().equals(node.getInstanceId())) {
                    Date end = new Date();
                    long different = end.getTime() - node.getGmtCreate().getTime();
                    long secondsInMilli = 1000;
                    long minutesInMilli = secondsInMilli * 60;
                    long hoursInMilli = minutesInMilli * 60;
                    long daysInMilli = hoursInMilli * 24;
                    long elapsedDays = different / daysInMilli;
                    different = different % daysInMilli;
                    long elapsedHours = different / hoursInMilli;
                    different = different % hoursInMilli;
                    long elapsedMinutes = different / minutesInMilli;
                    different = different % minutesInMilli;
                    long elapsedSeconds = different / secondsInMilli;
                    String str = String.format("%d天%d小时%d分钟%d秒", elapsedDays, elapsedHours, elapsedMinutes,
                            elapsedSeconds);
                    if (elapsedDays == 0 && elapsedHours == 0 && elapsedMinutes == 0) {
                        str = str.replace("0天0小时0分钟", "");
                    }
                    if (elapsedDays == 0 && elapsedHours == 0) {
                        str = str.replace("0天0小时", "");
                    }
                    if (elapsedDays == 0) {
                        str = str.replace("0天", "");
                    }
                    flowInstance.setDuration(str);
                    break;
                }
            }
        }
    }

    /**
     * 获取当前审批人信息
     *
     * @param flowInstance
     * @return
     */
    @Override
    public String getCurrentAuditors(FlowInstance flowInstance, RequestContext context) {
        BasicDomainModel<FlowDefinition, DefinitionModel> definition = flowDefinitionService.getDefinition(flowInstance.getDefId());
        if (null == definition) {
            return null;
        }
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("nodeState", FlowConstants.STATE_ACTIVE), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("t.instanceId", flowInstance.getId()), Sign.EQ));
        List<FlowNode> nodes = this.getNodes(context, new Pager(1, Integer.MAX_VALUE, params, null));
        String currentAuditors = "";
        for (FlowNode node : nodes) {
            BaseNodeModel baseNodeModel = definition.getModel().getNode(node.getNodeId());
            if (!(baseNodeModel instanceof FlowNodeModel)) {
                continue;
            }
            FlowNodeModel model = (FlowNodeModel) baseNodeModel;
            String auditorString = null;
            // 特定人
            if (!ListTool.isEmpty(model.getAssignee())) {
                List<String> names = smsUserService.listByIds(context, model.getAssignee()).stream().map(User::getName).collect(Collectors.toList());
                for (String name : names) {
                    if (StringUtils.isNotBlank(auditorString)) {
                        auditorString += ", " + name;
                    } else {
                        auditorString = name;
                    }
                }
            }
            // 角色
            if (!ListTool.isEmpty(model.getAssigneeRole())) {
                for (Long roleId : model.getAssigneeRole()) {
                    BocloudResult roleResult = smsRoleService.detail(roleId);
                    if (roleResult.isFailed()) {
                        return null;
                    }
                    String roleName = JSONObject.parseObject(roleResult.getData().toString(), Role.class).getName();
                    if (StringUtils.isNotBlank(auditorString)) {
                        auditorString += ", " + roleName + "(角色)";
                    } else {
                        auditorString = roleName + "(角色)";
                    }
                }
            }
            // 租户管理员
            if (model.getAssigneeTenantAdmin() != null && model.getAssigneeTenantAdmin()) {
                if (StringUtils.isNotBlank(auditorString)) {
                    auditorString += ", " + "租户管理员";
                } else {
                    auditorString = "租户管理员";
                }
            }
            // 项目经理
            if (model.getAssigneeProjectManager() != null && model.getAssigneeProjectManager()) {
                if (StringUtils.isNotBlank(auditorString)) {
                    auditorString += ", " + "项目经理";
                } else {
                    auditorString = "项目经理";
                }
            }
            if (StringUtils.isNotBlank(currentAuditors)) {
                currentAuditors += ", " + auditorString;
            } else {
                currentAuditors = auditorString;
            }
        }
        return currentAuditors;
    }

    @Override
    public String getDisplayName(RequestContext context) {
        if (RequestContext.Catalog.Tenant == context.getCatalog()) {
            Tenant tenant = smsTenantService.detail(context.getTarget());
            return tenant == null ? null : tenant.getName();
        } else {
            BocloudResult<User> result = smsUserService.detail(context.getTarget());
            return result.isSuccess() ? result.getData().getName() : null;
        }
    }

    @Override
    public long countTodo(RequestContext context) {
        Pager pager = new Pager();
        pager.setParams(new ArrayList<>(4));
        pager.getParams().add(new Param(MapTools.simpleMap("actorId", context.getTarget()), Sign.EQ));
        pager.getParams().add(new Param(MapTools.simpleMap("actorCatalog", RequestContext.Catalog.Tenant == context.getCatalog() ? RequestContext.Catalog.Tenant : RequestContext.Catalog.User), Sign.EQ));
        pager.getParams().add(new Param(MapTools.simpleMap("instanceState", FlowConstants.STATE_ACTIVE), Sign.EQ));
        pager.getParams().add(new Param(MapTools.simpleMap("nodeState", FlowConstants.STATE_ACTIVE), Sign.EQ));
        return this.getFlowInstanceRepository().count(pager);
    }

}
