package com.bocloud.fms.service.internal;

import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.utils.MapTools;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

@Component
public class ImsIpService implements InitializingBean {
    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public ImsIpService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private final Map<String, Object> headers = new HashMap<>(4);

    private static final Service SERVICE = Service.create(BoCloudService.IMS);

    /**
     * ip锁定
     *
     * @param ipString
     * @return
     */
    public BocloudResult preemptOrFreeIp(String ipString) {
        Map<String, Object> requestMap = MapTools.simpleMap("ipInfoString", ipString);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/ips/preemptOrFreeIp", BoCloudMethod.POST, headers, requestMap);
        return remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
