package com.bocloud.fms.service.internal;

import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.model.Pager;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * soc角色
 *
 * @author liuyuanyuan
 */
@Component
public class SmsRoleService implements InitializingBean {
    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public SmsRoleService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private final Map<String, Object> headers = new HashMap<>(4);

    /**
     * 根据id获取角色信息
     *
     * @param id 角色id
     * @return
     */
    public BocloudResult detail(Long id) {
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS),
                "/v1/roles/" + id, BoCloudMethod.GET, headers, null);
        return remoteService.invoke();
    }

    /**
     * 查询角色下用户
     *
     * @param roleId
     * @param pager
     * @return
     */
    public BocloudResult listUsers(Long roleId, Pager pager) {
        return serviceFactory.build(BoCloudService.SMS, "/v1/roles/" + roleId + "/users", BoCloudMethod.GET, headers, pager.toMap()).invoke();
    }

    @Override
    public void afterPropertiesSet() {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
