package com.bocloud.fms.service.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.fms.internel.entity.AccountSecurity;
import com.bocloud.fms.internel.entity.User;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author hufy
 */
@Component
public class SmsUserService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public SmsUserService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private final Map<String, Object> headers = new HashMap<>(4);

    private static final Service SERVICE = Service.create(BoCloudService.SMS);

    public BocloudResult<AccountSecurity> secKey(String apiKey) {
        Map<String, Object> params = MapTools.simpleMap("apiKey", apiKey);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/users/key", BoCloudMethod.GET, headers,
                params);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONObject.parseObject(data, AccountSecurity.class), "success");
    }

    /**
     * 获取用户详情
     *
     * @param id 用户id
     * @return
     */
    public BocloudResult<User> detail(Long id) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/users/" + id, BoCloudMethod.GET,
                headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONObject.parseObject(data, User.class), "success");
    }

    /**
     * 分页获取用户列表
     *
     * @param apiKey 请求api key
     * @param pager  分页对象
     * @return
     */
    public BocloudResult list(String apiKey, Pager pager) {
        headers.put(Common.API_KEY, apiKey);
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS), "/v1/users",
                BoCloudMethod.GET, headers, pager.toMap());
        return remoteService.invoke();
    }

    /**
     * 根据id集合获取对应用户集合
     *
     * @param context 请求对象
     * @param ids     用户id集合
     * @return
     */
    public List<User> listByIds(RequestContext context, List<Long> ids) {
        List<Param> params = new ArrayList<>();
        params.add(new Param(
                MapTools.simpleMap("id",
                        JSONObject.toJSONString(ids).substring(1, JSONObject.toJSONString(ids).length() - 1)),
                Sign.IN));
        params.add(new Param(MapTools.simpleMap("status", "NORMAL"), Sign.EQ));
        BocloudResult result = list(context.getApiKey(), new Pager(1, Integer.MAX_VALUE, params, null));
        return ((JSONObject) result.getData()).getJSONArray(Common.ROWS).toJavaList(User.class);
    }

    public BocloudResult<List<User>> listUsers(Long tenantId) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "listByTenant");
        object.put("tenantId", tenantId);
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/users", BoCloudMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONArray.parseArray(data, User.class), "success");
    }


    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

}
